require 'encrypted_yaml/decrypt'
require 'yaml'

module EncryptedYaml
  # represents an encrypted YAML file as a Hash
  class Configurator < Hash
    # @note The options param has some sane defaults:
    #   * for both key and iv, it will prefer the string blob over the filename
    #   * if neither the key/iv or filename is defined, it will default to a file named 'key' or 'iv'
    #     in the current directory
    # @param [String] filename path to the encrypted
    # @param [Hash] options optional settings for how to find the key and iv
    # @option options [String] :key the encryption key (as a string blob)
    # @option options [String] :iv the encryption iv (as a string blob)
    # @option options [String] :keyfile path to the key file
    # @option options [String] :ivfile path to the iv file    
    def initialize(filename, options = {})
      @key = if options[:key]
               options[:key]
             else
               keyfile = options[:keyfile] || File.dirname(filename) + '/key'
               File.read keyfile
             end
      
      @iv = if options[:iv]
              options[:iv]
            else
              ivfile = options[:ivfile] || File.dirname(filename) + '/iv'
              File.read ivfile
            end
      
      @filename = filename      
      load_config
    end

    # returns a new copy with the current values in the configuration file
    # @note does not reload the key or iv
    # @return [EncryptedConfig::Configurator] new copy with the config file reloaded
    def reload_config
      EncryptedYaml::Configurator.new(@filename, {
        :key      => @key,
        :iv       => @iv
      })
    end
    
    # reloads the configuration file
    # @note does not reload the key or iv
    def reload_config!
      self.clear
      load_config
    end

    private

    def load_config
      encrypted_data = File.read @filename
      decrypt = EncryptedYaml::Decrypt.new(@key, @iv)
      data = decrypt.decrypt encrypted_data
      self.replace YAML.load(data)
    end    
  end
end